package com.kernal.bankcard.utils;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.List;


import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.PixelFormat;
import android.graphics.Point;
import android.hardware.Camera;
import android.os.Build;
import android.util.Log;
import android.view.Display;
import android.view.SurfaceHolder;
import android.view.View;
import android.view.WindowManager;
import android.widget.Toast;

/**
 * Created by huangzhen on 2016/2/3.
 */
public class CameraSetting {
	static Camera.Parameters parameters;
	public static int srcWidth;
	public static int srcHeight;
	public static int preWidth, preHeight;
	public static int picWidth, picHeight;
	public static int surfaceWidth, surfaceHeight;
	static List<Camera.Size> list;
	private static boolean isShowBorder = false;
	private static Context context;
	public static String[] list_PicSize;
	public static int[] list_PicSize_Width;
	public static int[] list_PicSize_Height;
	public static int count;

	private CameraSetting(Context context) {
		this.context = context;
		setScreenSize(context);
	}

	private static CameraSetting single = null;

	// 静态工厂方法
	public static CameraSetting getInstance(Context context) {
		if (single == null) {
			single = new CameraSetting(context);
		}
		return single;
	}

	/**
	 * @return ${return_type} 返回类型
	 * @throws @Title:
	 *             关闭相机
	 * @Description: 释放相机资源
	 */
	public static Camera closeCamera(Camera camera) {
		try {
			if (camera != null) {
				camera.setPreviewCallback(null);
				camera.stopPreview();
				camera.release();
				camera = null;
			}
		} catch (Exception e) {
			Log.i("TAG", e.getMessage());
		}
		return camera;
	}

	/**
	 * @Title: ${enclosing_method} @Description: 打开闪光灯 @param camera
	 * 相机对象 @return void 返回类型 @throws
	 */
	public static void openCameraFlash(Camera camera) {
		if (camera == null)
			camera = Camera.open();
		Camera.Parameters parameters = camera.getParameters();
		List<String> flashList = parameters.getSupportedFlashModes();
		if (flashList != null && flashList.contains(Camera.Parameters.FLASH_MODE_TORCH)) {
			parameters.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
			camera.setParameters(parameters);
		} else {
			Toast.makeText(context, context.getString(context.getResources().getIdentifier("unsupportflash", "string", context.getPackageName())), Toast.LENGTH_SHORT).show();
		}

	};

	/**
	 * @Title: ${enclosing_method} @Description: 关闭闪光灯 @param camera
	 * 相机对象 @return void 返回类型 @throws
	 */
	public static void closedCameraFlash(Camera camera) {
		if (camera == null)
			camera = Camera.open();
		Camera.Parameters parameters = camera.getParameters();
		List<String> flashList = parameters.getSupportedFlashModes();
		if (flashList != null && flashList.contains(Camera.Parameters.FLASH_MODE_TORCH)) {
			parameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
			camera.setParameters(parameters);
		} else {
			Toast.makeText(context, context.getString(context.getResources().getIdentifier("unsupportflash", "string", context.getPackageName())), Toast.LENGTH_SHORT).show();
		}
	}

	/**
	 * @Title: ${enclosing_method} @Description: 设置图片大小 @param camera
	 * 相机对象 @return void 返回类型 @throws
	 */
	public static void setPicSize(Camera camera, int pic_width, int pic_height) {
		if (camera == null)
			camera = Camera.open();
		Camera.Parameters parameters = camera.getParameters();
		parameters.setPictureSize(pic_width, pic_height);
		camera.setParameters(parameters);
	}

	/**
	 * @param currentActivity
	 * @param sizes
	 *            最理想的预览分辨率的宽和高
	 * @param targetRatio
	 * @return 获得最理想的预览尺寸
	 */
	public static Camera.Size getOptimalPreviewSize(Activity currentActivity, List<Camera.Size> sizes,
			double targetRatio) {
		// Use a very small tolerance because we want an exact match.
		final double ASPECT_TOLERANCE = 0.001;
		if (sizes == null)
			return null;

		Camera.Size optimalSize = null;
		double minDiff = Double.MAX_VALUE;

		// Because of bugs of overlay and layout, we sometimes will try to
		// layout the viewfinder in the portrait orientation and thus get the
		// wrong size of mSurfaceView. When we change the preview size, the
		// new overlay will be created before the old one closed, which causes
		// an exception. For now, just get the screen size

		Display display = currentActivity.getWindowManager().getDefaultDisplay();
		int targetHeight = Math.min(display.getHeight(), display.getWidth());

		if (targetHeight <= 0) {
			// We don't know the size of SurfaceView, use screen height
			targetHeight = display.getHeight();
		}

		// Try to find an size match aspect ratio and size
		for (Camera.Size size : sizes) {
			double ratio = (double) size.width / size.height;
			if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE)
				continue;
			if (Math.abs(size.height - targetHeight) < minDiff) {
				optimalSize = size;
				minDiff = Math.abs(size.height - targetHeight);
			}
		}

		// Cannot find the one match the aspect ratio. This should not happen.
		// Ignore the requirement.
		if (optimalSize == null) {
			System.out.println("No preview size match the aspect ratio");
			minDiff = Double.MAX_VALUE;
			for (Camera.Size size : sizes) {
				if (Math.abs(size.height - targetHeight) < minDiff) {
					optimalSize = size;
					minDiff = Math.abs(size.height - targetHeight);
				}
			}
		}
		return optimalSize;
	}

	/**
	 * @param mDecorView
	 *            {tags} 设定文件
	 * @return ${return_type} 返回类型
	 * @throws @Title:
	 *             沉寂模式
	 * @Description: 隐藏虚拟按键
	 */
	@TargetApi(19)
	public static void hiddenVirtualButtons(View mDecorView) {
		if (Build.VERSION.SDK_INT >= 19) {
			mDecorView.setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_STABLE | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
					| View.SYSTEM_UI_FLAG_HIDE_NAVIGATION | View.SYSTEM_UI_FLAG_FULLSCREEN
					| View.SYSTEM_UI_FLAG_IMMERSIVE);
		}
	}

	/**
	 * @param previewCallback
	 *            相机界面预览回调函数
	 * @param surfaceHolder
	 *            相机界面的holder
	 * @param camera
	 *            相机对象
	 * @param cancelAutoFocus
	 *            是否取消自动对焦
	 * @param currentActivity
	 *            相机界面的Activity对象
	 * @param screenRatio
	 *            屏幕的宽高比
	 * @param srcList
	 *            拍照分辨率的宽高容器
	 * @return ${return_type} 返回类型
	 * @throws @Title:
	 *             ${enclosing_method}
	 * @Description: 设置相机的详细参数
	 */
	public void setCameraParameters(Camera.PreviewCallback previewCallback, SurfaceHolder surfaceHolder,
			Activity currentActivity, Camera camera, float screenRatio, List<Integer> srcList,
			boolean cancelAutoFocus) {
		Camera.Parameters parameters = camera.getParameters();
		// if (parameters.getSupportedFocusModes().contains(
		// parameters.FOCUS_MODE_CONTINUOUS_PICTURE)) {
		// parameters.setFocusMode(parameters.FOCUS_MODE_CONTINUOUS_PICTURE);
		// } else
		if (parameters.getSupportedFocusModes().contains(parameters.FOCUS_MODE_AUTO)) {
			parameters.setFocusMode(parameters.FOCUS_MODE_AUTO);
		}
		parameters.setPictureFormat(PixelFormat.JPEG);
		// if (srcList.get(0) >= 1280 && srcList.get(1) >= 720) {
		// parameters.setPictureSize(srcList.get(0), srcList.get(1));
		// System.out.println("srcwidth:" + srcList.get(0) + "--srcheight:"
		// + srcList.get(1));
		// }
		Camera.Size optimalPreviewSize = CameraSetting.getInstance(context).getOptimalPreviewSize(currentActivity,
				camera.getParameters().getSupportedPreviewSizes(), screenRatio);
		parameters.setPreviewSize(optimalPreviewSize.width, optimalPreviewSize.height);
		try {
			camera.setPreviewDisplay(surfaceHolder);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (cancelAutoFocus) {
			camera.cancelAutoFocus();
		}
		camera.setPreviewCallback(previewCallback);
		camera.setParameters(parameters);
		camera.startPreview();

	}

	@SuppressLint("NewApi")
	public static void setScreenSize(Context context) {
		int x, y;
		WindowManager wm = ((WindowManager) context.getSystemService(Context.WINDOW_SERVICE));
		Display display = wm.getDefaultDisplay();
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
			Point screenSize = new Point();
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
				display.getRealSize(screenSize);
				x = screenSize.x;
				y = screenSize.y;
			} else {
				display.getSize(screenSize);
				x = screenSize.x;
				y = screenSize.y;
			}
		} else {
			x = display.getWidth();
			y = display.getHeight();
		}

		srcWidth = x;
		srcHeight = y;
	}

	/**
	 * 获取设备的预览分辨率的宽和高
	 * 
	 * @param camera
	 */
	public static void getCameraPreParameters(Camera camera)

	{
		isShowBorder = false;
		// 荣耀七设备
		if ("PLK-TL01H".equals(Build.MODEL)) {
			preWidth = 1920;
			preHeight = 1080;
			return;
		}
		// 其他设备
		parameters = camera.getParameters();
		list = parameters.getSupportedPreviewSizes();
		float ratioScreen = (float) srcWidth / srcHeight;

		for (int i = 0; i < list.size(); i++) {
			float ratioPreview = (float) list.get(i).width / list.get(i).height;
			if (ratioScreen == ratioPreview) {// 判断屏幕宽高比是否与预览宽高比一样，如果一样执行如下代码
				if (list.get(i).width >= 1280 || list.get(i).height >= 720) {// 默认预览以1280*720为标准
					if (preWidth == 0 && preHeight == 0) {// 初始值
						preWidth = list.get(i).width;
						preHeight = list.get(i).height;
					}
					if (list.get(0).width > list.get(list.size() - 1).width) {
						// 如果第一个值大于最后一个值
						if (preWidth > list.get(i).width || preHeight > list.get(i).height) {
							// 当有大于1280*720的分辨率但是小于之前记载的分辨率，我们取中间的分辨率
							preWidth = list.get(i).width;
							preHeight = list.get(i).height;
						}
					} else {
						// 如果第一个值小于最后一个值
						if (preWidth < list.get(i).width || preHeight < list.get(i).height) {
							// 如果之前的宽度和高度大于等于1280*720，就不需要再筛选了
							if (preWidth >= 1280 || preHeight >= 720) {

							} else {
								// 为了找到合适的分辨率，如果preWidth和preHeight没有比1280*720大的就继续过滤
								preWidth = list.get(i).width;
								preHeight = list.get(i).height;
							}
						}
					}
				}
			}
		}
		// 说明没有找到程序想要的分辨率
		if (preWidth == 0 || preHeight == 0) {
			isShowBorder = true;
			preWidth = list.get(0).width;
			preHeight = list.get(0).height;
			for (int i = 0; i < list.size(); i++) {

				if (list.get(0).width > list.get(list.size() - 1).width) {
					// 如果第一个值大于最后一个值
					if (preWidth >= list.get(i).width || preHeight >= list.get(i).height) {
						// 当上一个选择的预览分辨率宽或者高度大于本次的宽度和高度时，执行如下代码:
						if (list.get(i).width >= 1280) {
							// 当本次的预览宽度和高度大于1280*720时执行如下代码
							preWidth = list.get(i).width;
							preHeight = list.get(i).height;

						}
					}
				} else {
					if (preWidth <= list.get(i).width || preHeight <= list.get(i).height) {
						if (preWidth >= 1280 || preHeight >= 720) {

						} else {
							// 当上一个选择的预览分辨率宽或者高度大于本次的宽度和高度时，执行如下代码:
							if (list.get(i).width >= 1280) {
								// 当本次的预览宽度和高度大于1280*720时执行如下代码
								preWidth = list.get(i).width;
								preHeight = list.get(i).height;

							}
						}

					}
				}
			}
		}
		// 如果没有找到大于1280*720的分辨率的话，取集合中的最大值进行匹配
		if (preWidth == 0 || preHeight == 0) {
			isShowBorder = true;
			if (list.get(0).width > list.get(list.size() - 1).width) {
				preWidth = list.get(0).width;
				preHeight = list.get(0).height;
			} else {
				preWidth = list.get(list.size() - 1).width;
				preHeight = list.get(list.size() - 1).height;
			}
		}
		if (isShowBorder) {
			if (ratioScreen > (float) preWidth / preHeight) {
				surfaceWidth = (int) (((float) preWidth / preHeight) * srcHeight);
				surfaceHeight = srcHeight;
			} else {
				surfaceWidth = srcWidth;
				surfaceHeight = (int) (((float) preHeight / preWidth) * srcWidth);
			}
		} else {
			surfaceWidth = srcWidth;
			surfaceHeight = srcHeight;
		}

	}

	/**
	 * 获取设备的拍照分辨率的宽和高
	 * 
	 * @param camera
	 */
	public static void getCameraPicParameters(Camera camera, float ratioScreen) {
		isShowBorder = false;
		parameters = camera.getParameters();
		list = parameters.getSupportedPictureSizes();
		list_PicSize_Width = new int[list.size()];
		list_PicSize_Height = new int[list.size()];
		count = 0;
		for (int i = 0; i < list.size(); i++) {
//			 System.out.println("图片分辨率："+list.get(i).width+";"+list.get(i).height);
			float ratioPicture = (float) list.get(i).width / list.get(i).height;
			if (ratioScreen <= ratioPicture+0.01 && ratioScreen >= ratioPicture-0.01) {// 判断屏幕宽高比是否与拍照宽高比一样，如果一样执行如下代码
				if (3500 >= list.get(i).width && list.get(i).width >= 1280 && list.get(i).height >= 960) {
					list_PicSize_Width[count] = list.get(i).width;
					list_PicSize_Height[count] = list.get(i).height;
					count++;
					if (list.get(i).width >= 1600 && list.get(i).height >= 1200) {// 默认预览以1600*1200为标准
						if (picWidth == 0 && picHeight == 0) {// 初始值
							picWidth = list.get(i).width;
							picHeight = list.get(i).height;
						}
						if (list.get(0).width > list.get(list.size() - 1).width) {
							// 如果第一个值大于最后一个值
							if (picWidth > list.get(i).width || picHeight > list.get(i).height) {
								// 当有大于1600*1200的分辨率但是小于之前记载的分辨率，我们取中间的分辨率
								picWidth = list.get(i).width;
								picHeight = list.get(i).height;
							}
						} else {
							// 如果第一个值小于最后一个值
							if (picWidth < list.get(i).width || picHeight < list.get(i).height) {
								// 如果之前的宽度和高度大于等于1600*1200，就不需要再筛选了
								if (picWidth >= 1600 || picHeight >= 1200) {

								} else {
									// 为了找到合适的分辨率，如果picWidth和picHeight没有比1600*1200大的就继续过滤
									picWidth = list.get(i).width;
									picHeight = list.get(i).height;
								}
							}
						}
					}

				}
			}
		}
		
		list_PicSize_Width = sort(list_PicSize_Width);
		list_PicSize_Height = sort(list_PicSize_Height);
		list_PicSize = new String[count];
		for(int i = 0;i<count;i++){
			list_PicSize [i] = list_PicSize_Width[i] + "*" +
					list_PicSize_Height[i];
//			System.out.println("i:"+i+";"+list_PicSize[i]+";"+list_PicSize_Width[i]);
		}
		// 说明没有找到程序想要的分辨率
		if (picWidth == 0 || picHeight == 0) {
			isShowBorder = true;
			picWidth = list.get(0).width;
			picHeight = list.get(0).height;
			for (int i = 0; i < list.size(); i++) {

				if (list.get(0).width > list.get(list.size() - 1).width) {
					// 如果第一个值大于最后一个值
					if (picWidth >= list.get(i).width || picHeight >= list.get(i).height) {
						// 当上一个选择的拍照分辨率宽或者高度大于本次的宽度和高度时，执行如下代码:
						if (list.get(i).width >= 1600) {
							// 当本次的预览宽度和高度大于1280*720时执行如下代码
							picWidth = list.get(i).width;
							picHeight = list.get(i).height;

						}
					}
				} else {
					if (picWidth <= list.get(i).width || picHeight <= list.get(i).height) {
						if (picWidth >= 1600 || picHeight >= 1200) {

						} else {
							// 当上一个选择的预览分辨率宽或者高度大于本次的宽度和高度时，执行如下代码:
							if (list.get(i).width >= 1600) {
								// 当本次的预览宽度和高度大于1280*720时执行如下代码
								picWidth = list.get(i).width;
								picHeight = list.get(i).height;

							}
						}

					}
				}
			}
		}
		// 如果没有找到大于1280*720的分辨率的话，取集合中的最大值进行匹配
		if (picWidth == 0 || picHeight == 0) {
			isShowBorder = true;
			if (list.get(0).width > list.get(list.size() - 1).width) {
				picWidth = list.get(0).width;
				picHeight = list.get(0).height;
			} else {
				picWidth = list.get(list.size() - 1).width;
				picHeight = list.get(list.size() - 1).height;
			}
		}
		if (isShowBorder) {
			if (ratioScreen > (float) picWidth / picHeight) {
				float rp = ratioScreen - ((float) preWidth / preHeight);
				// 如果误差在0.02之内，可以忽略
				if (rp <= 0.02) {
					surfaceWidth = srcWidth;
					surfaceHeight = srcHeight;
				} else {
					surfaceWidth = (int) (((float) preWidth / preHeight) * srcHeight);
					surfaceHeight = srcHeight;
				}
			} else {
				surfaceWidth = srcWidth;
				surfaceHeight = (int) (((float) picWidth / picHeight) * srcWidth);
			}
		} else {
			surfaceWidth = srcWidth;
			surfaceHeight = srcHeight;
		}
		// System.out.println("surfaceWidth:"+surfaceWidth+"--surfaceHeight:"+surfaceHeight);
		 
	}

	/**
	 * 给图片大小进行从大到小的排序
	 * 
	 * @param context
	 * @return
	 */
	private static int[] sort(int a[]) {
		for (int i = 0; i < a.length - 1; i++)
			for (int j = 0; j < a.length - i - 1; j++) {
				if (a[j] < a[j + 1]) // 这里是从大到小排序，如果是从小到大排序，只需将“<”换成“>”
				{
					int temp;
					temp = a[j];
					a[j] = a[j + 1];
					a[j + 1] = temp;
				}
			}
		return a;
	}

	// 获取NavigationBar的高度：
	public static int getNavigationBarHeight(Context context) {
		int navigationHeight = 0;
		Resources rs = context.getResources();
		int id = rs.getIdentifier("navigation_bar_height", "dimen", "android");
		if (id > 0 && checkDeviceHasNavigationBar(context)) {
			navigationHeight = rs.getDimensionPixelSize(id);
		}
		return navigationHeight;
	}

	// 获取是否存在NavigationBar：
	public static boolean checkDeviceHasNavigationBar(Context context) {
		boolean hasNavigationBar = false;
		Resources rs = context.getResources();
		int id = rs.getIdentifier("config_showNavigationBar", "bool", "android");
		if (id > 0) {
			hasNavigationBar = rs.getBoolean(id);
		}
		try {
			Class systemPropertiesClass = Class.forName("android.os.SystemProperties");
			Method m = systemPropertiesClass.getMethod("get", String.class);
			String navBarOverride = (String) m.invoke(systemPropertiesClass, "qemu.hw.mainkeys");
			if ("1".equals(navBarOverride)) {
				hasNavigationBar = false;
			} else if ("0".equals(navBarOverride)) {
				hasNavigationBar = true;
			}
		} catch (Exception e) {

		}

		return hasNavigationBar;

	}

}
